@@ -9,6 +9,16 @@ from ansiblelint.config import options
 from ansiblelint.file_utils import Lintable, normpath
 
 
+class StrictModeError(RuntimeError):
+    """Raise when we encounter a warning in strict mode."""
+
+    def __init__(
+        self, message: str = "Warning treated as error due to --strict option."
+    ):
+        """Initialize a StrictModeError instance."""
+        super().__init__(message)
+
+
 # pylint: disable=too-many-instance-attributes
 @functools.total_ordering
 class MatchError(ValueError):
@@ -53,11 +63,11 @@ class MatchError(ValueError):
         self.message = str(message or getattr(rule, "shortdesc", ""))
 
         # Safety measure to ensure we do not end-up with incorrect indexes
-        if linenumber == 0:
+        if linenumber == 0:  # pragma: no cover
             raise RuntimeError(
                 "MatchError called incorrectly as line numbers start with 1"
             )
-        if column == 0:
+        if column == 0:  # pragma: no cover
             raise RuntimeError(
                 "MatchError called incorrectly as column numbers start with 1"
             )
