@@ -29,6 +29,28 @@ from ansiblelint.yaml_utils import clean_json
 
 _logger = logging.getLogger(__name__)
 
+ignored_re = re.compile(
+    "|".join(
+        [
+            r"^parameters are mutually exclusive:",
+        ]
+    ),
+    flags=re.MULTILINE | re.DOTALL,
+)
+
+workarounds_drop_map = {
+    # https://github.com/ansible/ansible-lint/issues/3110
+    "ansible.builtin.copy": ["decrypt"],
+    # https://github.com/ansible/ansible-lint/issues/2824#issuecomment-1354337466
+    "ansible.builtin.service": ["daemon_reload"],
+    # Avoid: Unsupported parameters for (basic.py) module: cmd. Supported parameters include: _raw_params, _uses_shell, argv, chdir, creates, executable, removes, stdin, stdin_add_newline, strip_empty_ends.
+    "ansible.builtin.command": ["cmd"],
+}
+workarounds_inject_map = {
+    # https://github.com/ansible/ansible-lint/issues/2824
+    "ansible.builtin.async_status": {"_async_dir": "/tmp/ansible-async"}
+}
+
 
 @lru_cache
 def load_module(module_name: str) -> loader.PluginLoadContext:
@@ -76,6 +98,12 @@ class ArgsRule(AnsibleLintRule):
             for key, value in task["action"].items()
             if not key.startswith("__")
         }
+        if loaded_module.resolved_fqcn in workarounds_inject_map:
+            module_args.update(workarounds_inject_map[loaded_module.resolved_fqcn])
+        if loaded_module.resolved_fqcn in workarounds_drop_map:
+            for key in workarounds_drop_map[loaded_module.resolved_fqcn]:
+                if key in module_args:
+                    del module_args[key]
 
         with mock.patch.object(
             mock_ansible_module, "AnsibleModule", CustomAnsibleModule
@@ -131,6 +159,7 @@ class ArgsRule(AnsibleLintRule):
             except ValidationPassed:
                 return []
 
+    # pylint: disable=unused-argument
     def _sanitize_results(
         self, results: list[MatchError], module_name: str
     ) -> list[MatchError]:
@@ -138,46 +167,7 @@ class ArgsRule(AnsibleLintRule):
         sanitized_results = []
         for result in results:
             result_msg = result.message
-            if result_msg.startswith("Unsupported parameters"):
-                # cmd option is a special case in command module and after option validation is done.
-                if (
-                    "Unsupported parameters for (basic.py) module" in result_msg
-                    and module_name
-                    in ["command", "ansible.builtin.command", "ansible.legacy.command"]
-                ):
-                    continue
-                result.message = result_msg.replace("(basic.py)", f"{module_name}")
-            elif result_msg.startswith("missing required arguments"):
-                if (
-                    "missing required arguments: free_form" in result_msg
-                    and module_name
-                    in [
-                        "raw",
-                        "ansible.builtin.raw",
-                        "ansible.legacy.raw",
-                        "meta",
-                        "ansible.builtin.meta",
-                        "ansible.legacy.meta",
-                    ]
-                ):
-                    # free_form option is a special case in raw module hence ignore this error.
-                    continue
-                if (
-                    "missing required arguments: key_value" in result_msg
-                    and module_name
-                    in [
-                        "set_fact",
-                        "ansible.builtin.set_fact",
-                        "ansible.legacy.set_fact",
-                    ]
-                ):
-                    # handle special case for set_fact module with key and value
-                    continue
-            if "Supported parameters include" in result_msg and module_name in [
-                "set_fact",
-                "ansible.builtin.set_fact",
-                "ansible.legacy.set_fact",
-            ]:
+            if ignored_re.match(result_msg):
                 continue
             sanitized_results.append(result)
 
@@ -243,14 +233,13 @@ class ArgsRule(AnsibleLintRule):
 
 # testing code to be loaded only with pytest or when executed the rule file
 if "pytest" in sys.modules:
-
     from ansiblelint.runner import Runner  # pylint: disable=ungrouped-imports
 
     def test_args_module_fail() -> None:
         """Test rule invalid module options."""
         collection = RulesCollection()
         collection.register(ArgsRule())
-        success = "examples/playbooks/rule-args-module-fail-1.yml"
+        success = "examples/playbooks/rule-args-module-fail.yml"
         results = Runner(success, rules=collection).run()
         assert len(results) == 5
         assert results[0].tag == "args[module]"
@@ -268,6 +257,6 @@ if "pytest" in sys.modules:
         """Test rule valid module options."""
         collection = RulesCollection()
         collection.register(ArgsRule())
-        success = "examples/playbooks/rule-args-module-pass-1.yml"
+        success = "examples/playbooks/rule-args-module-pass.yml"
         results = Runner(success, rules=collection).run()
         assert len(results) == 0, results
