@@ -9,7 +9,7 @@ from typing import TYPE_CHECKING, Any
 
 import black
 import jinja2
-from ansible.errors import AnsibleError, AnsibleFilterError, AnsibleParserError
+from ansible.errors import AnsibleError, AnsibleParserError
 from ansible.parsing.yaml.objects import AnsibleUnicode
 from jinja2.exceptions import TemplateSyntaxError
 
@@ -17,6 +17,7 @@ from ansiblelint.constants import LINE_NUMBER_KEY
 from ansiblelint.file_utils import Lintable
 from ansiblelint.rules import AnsibleLintRule
 from ansiblelint.skip_utils import get_rule_skips_from_line
+from ansiblelint.text import has_jinja
 from ansiblelint.utils import parse_yaml_from_file, template
 from ansiblelint.yaml_utils import deannotate, nested_items_path
 
@@ -29,6 +30,22 @@ KEYWORDS_WITH_IMPLICIT_TEMPLATE = ("changed_when", "failed_when", "until", "when
 
 Token = namedtuple("Token", "lineno token_type value")
 
+ignored_re = re.compile(
+    "|".join(
+        [
+            r"^Object of type method is not JSON serializable",
+            r"^Unexpected templating type error occurred on",
+            r"^obj must be a list of dicts or a nested dict$",
+            r"^the template file (.*) could not be found for the lookup$",
+            r"could not locate file in lookup",
+            r"unable to locate collection",
+            r"^Error in (.*)is undefined$",
+            r"^Mandatory variable (.*) not defined.$",
+        ]
+    ),
+    flags=re.MULTILINE | re.DOTALL,
+)
+
 
 class JinjaRule(AnsibleLintRule):
     """Rule that looks inside jinja2 templates."""
@@ -51,85 +68,96 @@ class JinjaRule(AnsibleLintRule):
         """Generate error message."""
         return self._tag2msg[tag].format(value=value, reformatted=reformatted)
 
-    # pylint: disable=too-many-branches
+    # pylint: disable=too-many-branches,too-many-locals
     def matchtask(  # noqa: C901
         self, task: dict[str, Any], file: Lintable | None = None
-    ) -> bool | str | MatchError:
-        for key, v, _ in nested_items_path(task):
-            if isinstance(v, str):
-                try:
-                    template(
-                        basedir=file.dir if file else ".",
-                        value=v,
-                        variables=deannotate(task.get("vars", {})),
-                        fail_on_error=True,  # we later decide which ones to ignore or not
-                    )
-                # ValueError RepresenterError
-                except AnsibleError as exc:
-                    bypass = False
-                    orig_exc = exc.orig_exc if getattr(exc, "orig_exc", None) else exc
-                    match = self._ansible_error_re.match(
-                        getattr(orig_exc, "message", str(orig_exc))
-                    )
-                    if (
-                        isinstance(exc, AnsibleFilterError)
-                        or "unable to locate collection" in orig_exc.message
-                    ):
-                        bypass = True
-                    elif re.match(
-                        r"^the template file (.*) could not be found for the lookup$",
-                        orig_exc.message,
-                    ) or re.match(r"could not locate file in lookup", orig_exc.message):
-                        bypass = True
-                    elif isinstance(orig_exc, AnsibleParserError):
-                        # "An unhandled exception occurred while running the lookup plugin '...'. Error was a <class 'ansible.errors.AnsibleParserError'>, original message: Invalid filename: 'None'. Invalid filename: 'None'"
-
-                        # An unhandled exception occurred while running the lookup plugin 'template'. Error was a <class 'ansible.errors.AnsibleError'>, original message: the template file ... could not be found for the lookup. the template file ... could not be found for the lookup
-
-                        # ansible@devel (2.14) new behavior:
-                        # AnsibleError(TemplateSyntaxError): template error while templating string: Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'. String: Foo {{ buildset_registry.host | ipwrap }}. Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'
-                        bypass = True
-                    elif (
-                        isinstance(orig_exc, (AnsibleError, TemplateSyntaxError))
-                        and match
-                    ):
-                        error = match.group("error")
-                        detail = match.group("detail")
-                        # string = match.group("string")
-                        if error.startswith("template error while templating string"):
-                            bypass = False
-                        elif detail.startswith("unable to locate collection"):
-                            _logger.debug("Ignored AnsibleError: %s", exc)
-                            bypass = True
-                        else:
-                            bypass = False
-                    elif re.match(r"^lookup plugin (.*) not found$", exc.message):
-                        # lookup plugin 'template' not found
-                        bypass = True
-
-                    # AnsibleFilterError: 'obj must be a list of dicts or a nested dict'
-                    # AnsibleError: template error while templating string: expected token ':', got '}'. String: {{ {{ '1' }} }}
-                    # AnsibleError: template error while templating string: unable to locate collection ansible.netcommon. String: Foo {{ buildset_registry.host | ipwrap }}
-                    if not bypass:
-                        return self.create_matcherror(
-                            message=str(exc),
-                            linenumber=task[LINE_NUMBER_KEY],
-                            filename=file,
-                            tag=f"{self.id}[invalid]",
+    ) -> list[MatchError]:
+        result = []
+        try:
+            for key, v, path in nested_items_path(
+                task,
+                ignored_keys=("block", "ansible.builtin.block", "ansible.legacy.block"),
+            ):
+                if isinstance(v, str):
+                    try:
+                        template(
+                            basedir=file.dir if file else ".",
+                            value=v,
+                            variables=deannotate(task.get("vars", {})),
+                            fail_on_error=True,  # we later decide which ones to ignore or not
                         )
+                    # ValueError RepresenterError
+                    except AnsibleError as exc:
+                        bypass = False
+                        orig_exc = (
+                            exc.orig_exc if getattr(exc, "orig_exc", None) else exc
+                        )
+                        orig_exc_message = getattr(orig_exc, "message", str(orig_exc))
+                        match = self._ansible_error_re.match(
+                            getattr(orig_exc, "message", str(orig_exc))
+                        )
+                        if ignored_re.match(orig_exc_message):
+                            bypass = True
+                        elif isinstance(orig_exc, AnsibleParserError):
+                            # "An unhandled exception occurred while running the lookup plugin '...'. Error was a <class 'ansible.errors.AnsibleParserError'>, original message: Invalid filename: 'None'. Invalid filename: 'None'"
 
-                reformatted, details, tag = self.check_whitespace(
-                    v, key=key, lintable=file
-                )
-                if reformatted != v:
-                    return self.create_matcherror(
-                        message=self._msg(tag=tag, value=v, reformatted=reformatted),
-                        linenumber=task[LINE_NUMBER_KEY],
-                        details=details,
-                        filename=file,
-                        tag=f"{self.id}[{tag}]",
+                            # An unhandled exception occurred while running the lookup plugin 'template'. Error was a <class 'ansible.errors.AnsibleError'>, original message: the template file ... could not be found for the lookup. the template file ... could not be found for the lookup
+
+                            # ansible@devel (2.14) new behavior:
+                            # AnsibleError(TemplateSyntaxError): template error while templating string: Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'. String: Foo {{ buildset_registry.host | ipwrap }}. Could not load "ipwrap": 'Invalid plugin FQCN (ansible.netcommon.ipwrap): unable to locate collection ansible.netcommon'
+                            bypass = True
+                        elif (
+                            isinstance(orig_exc, (AnsibleError, TemplateSyntaxError))
+                            and match
+                        ):
+                            error = match.group("error")
+                            detail = match.group("detail")
+                            # string = match.group("string")
+                            if error.startswith(
+                                "template error while templating string"
+                            ):
+                                bypass = False
+                            elif detail.startswith("unable to locate collection"):
+                                _logger.debug("Ignored AnsibleError: %s", exc)
+                                bypass = True
+                            else:
+                                bypass = False
+                        elif re.match(r"^lookup plugin (.*) not found$", exc.message):
+                            # lookup plugin 'template' not found
+                            bypass = True
+
+                        # AnsibleFilterError: 'obj must be a list of dicts or a nested dict'
+                        # AnsibleError: template error while templating string: expected token ':', got '}'. String: {{ {{ '1' }} }}
+                        # AnsibleError: template error while templating string: unable to locate collection ansible.netcommon. String: Foo {{ buildset_registry.host | ipwrap }}
+                        if not bypass:
+                            result.append(
+                                self.create_matcherror(
+                                    message=str(exc),
+                                    linenumber=_get_error_line(task, path),
+                                    filename=file,
+                                    tag=f"{self.id}[invalid]",
+                                )
+                            )
+                            continue
+                    reformatted, details, tag = self.check_whitespace(
+                        v, key=key, lintable=file
                     )
-        return False
+                    if reformatted != v:
+                        result.append(
+                            self.create_matcherror(
+                                message=self._msg(
+                                    tag=tag, value=v, reformatted=reformatted
+                                ),
+                                linenumber=_get_error_line(task, path),
+                                details=details,
+                                filename=file,
+                                tag=f"{self.id}[{tag}]",
+                            )
+                        )
+        except Exception as exc:
+            _logger.info("Exception in JinjaRule.matchtask: %s", exc)
+            raise
+        return result
 
     def matchyaml(self, file: Lintable) -> list[MatchError]:
         """Return matches for variables defined in vars files."""
@@ -175,6 +203,8 @@ class JinjaRule(AnsibleLintRule):
 
         self.env.lstrip_blocks = False
         self.env.trim_blocks = False
+        self.env.autoescape = True
+        self.env.newline_sequence = "\n"
         tokens = [
             Token(lineno=t[0], token_type=t[1], value=t[2]) for t in self.env.lex(text)
         ]
@@ -244,6 +274,10 @@ class JinjaRule(AnsibleLintRule):
             implicit = True
             text = cook(text, implicit=implicit)
 
+        # don't try to lex strings that have no jinja inside them
+        if not has_jinja(text):
+            return text, "", "spacing"
+
         expr_str = None
         expr_type = None
         verb_skipped = True
@@ -329,7 +363,7 @@ if "pytest" in sys.modules:  # noqa: C901
     @pytest.fixture(name="error_expected_lines")
     def fixture_error_expected_lines() -> list[int]:
         """Return list of expected error lines."""
-        return [31, 34, 37, 40, 43, 46, 72]
+        return [33, 36, 39, 42, 45, 48, 74]
 
     # 21 68
     @pytest.fixture(name="lint_error_lines")
@@ -557,6 +591,22 @@ if "pytest" in sys.modules:  # noqa: C901
                 "spacing",
                 id="43",
             ),
+            # https://github.com/ansible/ansible-lint/pull/3057
+            # since jinja 3.0.0, \r is converted to \n if the string has jinja in it
+            pytest.param(
+                "{{ 'foo' }}\r{{ 'bar' }}",
+                "{{ 'foo' }}\n{{ 'bar' }}",
+                "spacing",
+                id="44",
+            ),
+            # if we do not have any jinja constructs, we should keep original \r
+            # to match ansible behavior
+            pytest.param(
+                "foo\rbar",
+                "foo\rbar",
+                "spacing",
+                id="45",
+            ),
         ),
     )
     def test_jinja(text: str, expected: str, tag: str) -> None:
@@ -621,9 +671,13 @@ if "pytest" in sys.modules:  # noqa: C901
         collection.register(JinjaRule())
         success = "examples/playbooks/rule-jinja-invalid.yml"
         errs = Runner(success, rules=collection).run()
-        assert len(errs) == 1
-        assert errs[0].tag == "jinja[invalid]"
+        assert len(errs) == 2
+        assert errs[0].tag == "jinja[spacing]"
         assert errs[0].rule.id == "jinja"
+        assert errs[0].linenumber == 9
+        assert errs[1].tag == "jinja[invalid]"
+        assert errs[1].rule.id == "jinja"
+        assert errs[1].linenumber == 9
 
     def test_jinja_valid() -> None:
         """Tests our ability to parse jinja, even when variables may not be defined."""
@@ -632,3 +686,16 @@ if "pytest" in sys.modules:  # noqa: C901
         success = "examples/playbooks/rule-jinja-valid.yml"
         errs = Runner(success, rules=collection).run()
         assert len(errs) == 0
+
+
+def _get_error_line(task: dict[str, Any], path: list[str | int]) -> int:
+    """Return error line number."""
+    line = task[LINE_NUMBER_KEY]
+    ctx = task
+    for _ in path:
+        ctx = ctx[_]
+        if LINE_NUMBER_KEY in ctx:
+            line = ctx[LINE_NUMBER_KEY]
+    if not isinstance(line, int):
+        raise RuntimeError("Line number is not an integer")
+    return line
